#!/bin/sh

x=11
while [ $x -gt 0 ]
do
	check_mount=$(cat /proc/mounts | grep "/etc" | wc -l)
	if [ ${check_mount} -eq 0 ]; then
		break;
	else
		echo "umount /etc times=$x"
		umount /etc
	fi
	x=`expr $x - 1`;
done

#Check update version of factory firmware
if [ -f /mnt/*factory*bin ];then
	have_fw="1"
	fw_name=`ls /mnt/*factory*bin`
fi

if [ ${have_fw} = "1" ]; then
    #Checksum of factory firmware
    cd /mnt
    /usr/bin/md5sum -c *factory*bin*sum
    if [ $? -eq 1 ]; then
	echo "Failed of factory firmware checksum!"
	cd -
        exit 1
    else
	cd -
	echo "Verified the factory firmware existed."
	/bin/dd if=${fw_name} of=/tmp/updver bs=1k count=128 skip=43520
	/bin/sync
	chk_u=`grep "UBOOT_VER" /tmp/updver | wc -l`
	chk_k=`grep "KERNEL_VER" /tmp/updver | wc -l`
	chk_r=`grep "RAMDISK_VER" /tmp/updver | wc -l`
	chk_e=`grep "ETC_VER" /tmp/updver | wc -l`
	if [ $chk_u -gt 0 ] && [ $chk_k -gt 0 ] && [ $chk_r -gt 0 ] && [ $chk_e -gt 0 ]; then
	    echo "Verified the update version correctly."
	    MSG="/tmp/message"
	    DBG="/tmp/debug"

	    # Write flash image from factory firmware

	    # Check if there is USB_upgrade in rc.sysinit of ramdisk.
	    # Do these steps for the 1.00.00 with 64kb erase size of JFFS2.
	    # if no exist, flashcp ramdisk only and reboot; 
	    # else flashcp factorty firmware image (uboot, kernel, ramdisk, jffs2, etc_img, updver).
	    rsiuu_exist=`grep "usb_upgrade" /etc/rc.d/rc.sysinit | wc -l`
	    if [ $rsiuu_exist -eq 0 ]; then
		# "usb_upgrade" script doesn't exist in rc.sysinit
		echo  "USB upgrade script doesn't exist in rc.sysinit."
		#Lock for upgrade
		/opt/bin/lockfile /var/lock/upgrade.lock

		echo "Updating ramdisk..."
		# Write ramdisk 
		/bin/dd if=${fw_name} of=/tmp/rd bs=1k count=30720 skip=4352
		/bin/sync
		/sbin/flashcp -v /tmp/rd /dev/mtd2
		if [ $? -eq 1 ]; then
		    Reason="Upgrade ramdisk fail!"
		    /app/bin/logevent/event 145 "${Reason}"
		    echo "${Reason}" > "${MSG}"
		    echo "${Reason}" >> "${DBG}"
		    exit 1
		fi
		/bin/sync
		
		#Unlock
		rm -f /var/lock/upgrade.lock

		echo "Updated ramdisk successfully, restart system!"
		#Restart system for update rc.sysinit file
		/sbin/reboot
	    else
		# "usb_upgrade" script exist in rc.sysinit
		echo  "USB upgrade script exist in rc.sysinit."

		#Lock for upgrade
		/opt/bin/lockfile /var/lock/upgrade.lock

		echo "Updating u-boot..."
		# Write Uboot
		/bin/dd if=${fw_name} of=/tmp/uboot bs=1k count=256
		/bin/sync
		/sbin/flashcp -v /tmp/uboot /dev/mtd0
                if [ $? -eq 1 ]; then
                        Reason="Upgrade u-boot fail!"
                        /app/bin/logevent/event 145 "${Reason}"
                        echo "${Reason}" > "${MSG}"
                        echo "${Reason}" >> "${DBG}"
                        exit 1
                fi
		/bin/sync
	   
		echo "Updating kernel..."
		#Write Kernel	
		/bin/dd if=${fw_name} of=/tmp/kernel bs=1k count=4096 skip=256
		/bin/sync
		/sbin/flashcp -v /tmp/kernel /dev/mtd1
		if [ $? -eq 1 ]; then
                        Reason="Upgrade kernel fail!"
                        /app/bin/logevent/event 145 "${Reason}"
                        echo "${Reason}" > "${MSG}"
                        echo "${Reason}" >> "${DBG}"
                        exit 1
                fi
	
		echo "Updating ramdisk..."
		# Write ramdisk 
		/bin/dd if=${fw_name} of=/tmp/rd bs=1k count=30720 skip=4352
		/bin/sync
		/sbin/flashcp -v /tmp/rd /dev/mtd2
		if [ $? -eq 1 ]; then
		    Reason="Upgrade ramdisk fail!"
		    /app/bin/logevent/event 145 "${Reason}"
		    echo "${Reason}" > "${MSG}"
		    echo "${Reason}" >> "${DBG}"
		    exit 1
		fi
		/bin/sync

		echo "Updating etc JFFS2..."
		#Write etc JFFS2
		/bin/dd if=${fw_name} of=/tmp/jffs2 bs=1k count=8192 skip=35072
		/bin/sync
		/sbin/flashcp -v /tmp/jffs2 /dev/mtd3
		if [ $? -eq 1 ]; then
			Reason="Upgrade etc JFFS2 fail!"
			/app/bin/logevent/event 145 "${Reason}"
			echo "${Reason}" > "${MSG}"
			echo "${Reason}" >> "${DBG}"
			exit 1
		fi
		/bin/sync
	
		echo "Updating etc backup image..."
		#Write etc backup iamge
		/bin/dd if=${fw_name} of=/tmp/etcimg bs=1k count=256 skip=43264
		/bin/sync
		/sbin/flashcp -v /tmp/etcimg /dev/mtd4
		if [ $? -eq 1 ]; then
			Reason="Upgrade etc bakup fail!"
			/app/bin/logevent/event 145 "${Reason}"
			echo "${Reason}" > "${MSG}"
			echo "${Reason}" >> "${DBG}"
			exit 1
		fi
		/bin/sync

		echo "Updating firmware update version..."
		#Write upgrade version
		/sbin/flashcp -v /tmp/updver /dev/mtd5
		if [ $? -eq 1 ]; then
			Reason="Upgrade upgrade version fail!"
			/app/bin/logevent/event 145 "${Reason}"
			echo "${Reason}" > "${MSG}"
			echo "${Reason}" >> "${DBG}"
			exit 1
		fi
		/bin/sync

		#Unlock
		rm -f /var/lock/upgrade.lock
		
		echo "Updated factory firwmare successfully, power off!"
		#System power off
		/app/bin/sys_halt
	    fi 

	fi #Update version existed.
    fi #Checksum factory firmware.
fi #Factory firmware existed.








